@@ -371,3 +371,26 @@ def getHubSiteConfig(hubname, sectionName, valueName):
 	
 	return rv
 	
+
+def getCMSversion():
+
+	defaultSite = hubzero.config.hubzerositeconfig.getHubDefaultSite()
+	wwwDocRoot = hubzero.config.hubzerositeconfig.getHubzeroConfigOption(defaultSite, 'documentroot')
+	fileName = wwwDocRoot + "/libraries/cms.php"
+
+	if os.path.isfile(wwwDocRoot + "/libraries/cms.php"):
+		f = open(fileName, 'r')
+		fileText =  f.read()
+		p = re.compile(r"HVERSION['\"]+\s*,\s*['\"]+\s*(.*?)\s*['\"]+", re.MULTILINE)
+		m = p.search(fileText)
+		if not m:
+			# 1.2.1 is the only version that will have this file withtout the version section in it
+			return "1.2.1"
+		else:
+			# return the specified version
+			return m.group(1)
+	else:
+		# if file doesn't exist we know we're 1.2.0 or older
+		return "1.2.0"
+
+
