@@ -33,6 +33,8 @@ import hubzero.config.ldapconfig
 import hubzero.config.webconfig
 import hubzero.data.db
 import ldap
+import os
+import subprocess
 
 
 def groupCleanup(db, groupname):
@@ -139,11 +141,14 @@ def del_jos_xgroups_members(gidNumber, uidNumber):
 	_del_jos_xgroups_members(db, gidNumber, uidNumber)
 
 
-def add_ldap_group(cn, groupDescription, gidNumber, members = {}):
+def add_ldap_group(cn, groupDescription, gidNumber=0, members = {}):
 	hubLDAPAcctMgrDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerdn")
 	hubLDAPAcctMgrPW = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerpw")
 	hubLDAPBaseDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_basedn")
 
+	if not gidNumber:
+		gidNumber = _getnextLDAPGIDNumber()
+
 	ldapServer = hubzero.config.webconfig.getComponentParam("com_system", "ldap_primary")
 	l = ldap.initialize(ldapServer)
 	l.simple_bind_s(hubLDAPAcctMgrDN, hubLDAPAcctMgrPW)
@@ -155,9 +160,21 @@ def add_ldap_group(cn, groupDescription, gidNumber, members = {}):
 	attrs['gidNumber'] = str(gidNumber)
 	attrs['description'] = groupDescription
 	attrs['memberuid'] = members
-	
+
 	ldif = ldap.modlist.addModlist(attrs)
-	l.add_s(dn, ldif)
+
+	try:
+		l.add_s(dn, ldif)
+	except ldap.ALREADY_EXISTS:
+		print "Group " + cn + " already exists"
+		return 0
+
+	# to avoid delays in cache
+	#os.system("/etc/init.d/nscd restart")
+	#os.system("/etc/init.d/sssd restart")
+
+	return gidNumber
+
 
 def delete_ldap_group(groupName):
 	hubLDAPAcctMgrDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerdn")
@@ -169,19 +186,27 @@ def delete_ldap_group(groupName):
 	l.simple_bind_s(hubLDAPAcctMgrDN, hubLDAPAcctMgrPW)
 
 	dn = "cn=" + groupName + ",ou=groups," + hubLDAPBaseDN
-	
+	print "delete: " + dn	
 	try:
 		l.delete_s(dn)
 	except ldap.NO_SUCH_OBJECT:
-		pass 
+		print "group not found"
 
 	
-def _insert_jos_xgroups(db, name, description='', grouptype=0):
-	sql =  "INSERT INTO jos_xgroups ( `cn`, `description`, `published`, `type`, `public_desc`, `private_desc`, `restrict_msg`, "
-	sql += "`join_policy`, `discussion_email_autosubscribe`, `logo`, `overview_type`, `overview_content`, `plugins`, `created`,"
-	sql += "`created_by`, `params`) " 
-	sql += "VALUES (" + '%s,'*15 + "%s );" 
-	data = (name, description, 0, int(grouptype), '', '', '', 1, 0, '', None, None, None, datetime.datetime.now(), None, None)
+def _insert_jos_xgroups(db, name, description='', grouptype=0, gidNumber=0):
+
+	if not gidNumber:
+		sql =  "INSERT INTO jos_xgroups ( `cn`, `description`, `published`, `type`, `public_desc`, `private_desc`, `restrict_msg`, "
+		sql += "`join_policy`, `discussion_email_autosubscribe`, `logo`, `overview_type`, `overview_content`, `plugins`, `created`,"
+		sql += "`created_by`, `params`) " 
+		sql += "VALUES (" + '%s,'*15 + "%s );" 
+		data = (name, description, 0, int(grouptype), '', '', '', 1, 0, '', None, None, None, datetime.datetime.now(), None, None)
+	else:
+		sql =  "INSERT INTO jos_xgroups ( `gidNumber`, `cn`, `description`, `published`, `type`, `public_desc`, `private_desc`, `restrict_msg`, "
+		sql += "`join_policy`, `discussion_email_autosubscribe`, `logo`, `overview_type`, `overview_content`, `plugins`, `created`,"
+		sql += "`created_by`, `params`) " 
+		sql += "VALUES (" + '%s,'*16 + "%s );" 
+		data = (gidNumber, name, description, 0, int(grouptype), '', '', '', 1, 0, '', None, None, None, datetime.datetime.now(), None, None)
 
 	gidNumber = db.query_lastrowid(sql, data)
 
@@ -212,11 +237,11 @@ def insert_jos_xgroups_members(userName, groupName):
 	_insert_jos_xgroups_members(db, gidNumber, uidNumber)
 
 
-def delhubgroup(hubname, groupName):
+def delhubgroup(groupName):
 	# grab config options
-	dbPW = hubzero.config.webconfig.getWebConfigDBPassword(hubname)
-	dbUserName = hubzero.config.webconfig.getWebConfigDBUsername(hubname)
-	dbName = hubzero.config.webconfig.getWebConfigDBName(hubname)
+	dbPW = hubzero.config.webconfig.getWebConfigDBPassword()
+	dbUserName = hubzero.config.webconfig.getWebConfigDBUsername()
+	dbName = hubzero.config.webconfig.getWebConfigDBName()
 
 	db =  hubzero.data.db.MySQLConnection("localhost", dbUserName, dbName, dbPW)
 	delete_ldap_group(groupName)
@@ -238,6 +263,18 @@ def addhubgroup(groupName, description='', type=0):
 	return gidNumber	
 
 
+def addhubgroupdbonly(groupName, gid=0, description='', type=0):
+
+	# grab config options
+	dbPW = hubzero.config.webconfig.getWebConfigDBPassword()
+	dbUserName = hubzero.config.webconfig.getWebConfigDBUsername()
+	dbName = hubzero.config.webconfig.getWebConfigDBName()
+	db =  hubzero.data.db.MySQLConnection("localhost", dbName, dbUserName, dbPW)
+
+        gidNumber = _insert_jos_xgroups(db, groupName, description, type, gid)
+	return gidNumber	
+
+
 def _ldapGroupExists(groupName):
 
 	hubLDAPAcctMgrDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerdn")
@@ -256,6 +293,56 @@ def _ldapGroupExists(groupName):
 		return True
 
 
+def _getnextLDAPGIDNumber(start = 1000, end = 99999999):
+	# query ldap directly for next gidNumber
+	hubLDAPAcctMgrDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerdn")
+	hubLDAPAcctMgrPW = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerpw")
+	hubLDAPBaseDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_basedn")
+	siteSuffix = hubzero.config.ldapconfig.getLDAPSiteSuffix()
+	
+	ldapServer = hubzero.config.webconfig.getComponentParam("com_system", "ldap_primary")
+	l = ldap.initialize(ldapServer)
+	l.simple_bind_s(hubLDAPAcctMgrDN, hubLDAPAcctMgrPW)
+
+	nextGIDNumber = start
+
+	# get all groups from ldap
+	try:
+		r = l.search_s(
+			"ou=groups," + siteSuffix, 
+			ldap.SCOPE_SUBTREE, 
+			'(objectClass=posixGroup)',   
+			['cn', 'gidNumber'])
+
+	except ldap.NO_SUCH_OBJECT:
+		return low
+
+	# loop through all the gidNumbers
+	for dn,entry in r:
+		gidNumber = int(entry['gidNumber'][0])
+
+		if gidNumber > nextGIDNumber:
+			nextGIDNumber = gidNumber
+
+		if gidNumber > end:
+			raise Exception('No open gidNumbers available')
+
+	return nextGIDNumber+1
+
+
+def _getnextLdapGID():
+
+	# pipe a `getent group` call into a awk used to analyze for next groupid
+	# less desirable than _getnextLDAPGIDNumber due to caching issues causing
+	# the getent group call to not reflect new groups just added
+
+	p1 = subprocess.Popen(['getent', 'group'], stdout=subprocess.PIPE)
+	s = '{uid[$3]=1}END{for(x=1000; x<=65000; x++) {if(uid[x] != ""){}else{print x; exit;}}}'
+	p2 = subprocess.Popen(['awk', '-F:', s], stdin=p1.stdout, stdout=subprocess.PIPE)
+	n = p2.communicate()
+	return n[0].strip()
+
+
 def addusertogroup_ldap(userName, groupName):
 
 	hubLDAPAcctMgrDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerdn")
@@ -305,7 +392,6 @@ def deleteuserfromgroup(userName, groupName):
 	deleteuserfromgroup_ldap(userName, groupName)
 
 
-
 def getUserGroups(userName):
 	dbPW = hubzero.config.webconfig.getWebConfigDBPassword()
 	dbUserName = hubzero.config.webconfig.getWebConfigDBUsername()
@@ -325,6 +411,7 @@ def getUserGroups(userName):
 
 	return userGroupMembershipList
 
+
 def userInGroup(userName, groupName):
 	groups = getUserGroups(userName)
 
@@ -336,9 +423,11 @@ def userInGroup(userName, groupName):
 def syncgroup(group, hubname = "", purgeorphans = True, addgroupmemberships = True, verbose = False, debug = False):
 	_syncgroups(hubname,purgeorphans,addgroupmemberships,group,verbose,debug)
 
+
 def syncgroups(hubname = "", purgeorphans = True, addgroupmemberships = True, verbose = False, debug = False):
     _syncgroups(hubname,purgeorphans,addgroupmemberships,'*',verbose,debug)
 
+
 def _syncgroups(hubname = "", purgeorphans = True, addgroupmemberships = True, group = '*', verbose = False, debug = False):
 	
 	dbPW = hubzero.config.webconfig.getWebConfigDBPassword()
@@ -461,4 +550,4 @@ def _delete_ldap_users(args):
 
 		except ldap.NO_SUCH_OBJECT:
 			print "No users in ou=users"
-	
\ No newline at end of file
+	
